@@ -19,23 +19,20 @@ Constructor. Accepts a string to be parsed
 =cut
 
 sub new {
-    my $class = shift;
-    my $line = shift;
-    # don't accept undefined values
-    die "Missing input!" unless (defined $line);
-    die "Too many arguments, I accept only a single string" if @_;
-
+    my ($class, %args) = @_;
     my $self = {
-                rawline => $line,
-                block => "",      # the block it says to belog
-                type => "null", # the type
-                string => "",      # the string
-                removed => "", # the portion of the string removed
+                rawline => '',
+                block => '',      # the block it says to belog
+                type => 'null', # the type
+                string => '',      # the string
+                removed => '', # the portion of the string removed
                };
+    foreach my $accessor (keys %$self) {
+        if (exists $args{$accessor} and defined $args{$accessor}) {
+            $self->{$accessor} = $args{$accessor};
+        }
+    }
     bless $self, $class;
-    # initialize it
-    $self->_parse_string;
-    return $self;
 }
 
 =head3 rawline
@@ -139,153 +136,6 @@ sub indentation {
     return length($self->removed);
 }
 
-sub _block_re {
-    my $self = shift;
-    return qr{(
-                 biblio   |
-                 play     |
-                 comment  |
-                 verse    |
-                 center   |
-                 right    |
-                 example  |
-                 quote    |
-                 li | ul | ola | olA | oli | olI | oln # these are private
-
-             )}x
-}
-
-sub _parse_string {
-    my $self = shift;
-    my $l = $self->rawline;
-    my $blockre = $self->_block_re;
-    # null line is default, do nothing
-    if ($l =~ m/^[\n\t ]*$/s) {
-        # do nothing, already default
-        $self->removed($l);
-    }
-    elsif ($l =~ m!^(<($blockre)>\s*)$!s) {
-        $self->type("startblock");
-        $self->removed($1);
-        $self->block($2);
-    }
-    elsif ($l =~ m!^(</($blockre)>\s*)$!s) {
-        $self->type("stopblock");
-        $self->removed($1);
-        $self->block($2);
-    }
-    # headers
-    elsif ($l =~ m!^((\*{1,5}) )(.+)$!s) {
-        $self->type("h" . length($2));
-        $self->removed($1);
-        $self->string($3);
-    }
-    elsif ($l =~ m/^( +\- +)(.*)/s) {
-        $self->type("li");
-        $self->removed($1);
-        $self->string($2);
-        $self->block("ul");
-    }
-    elsif ($l =~ m/^(\s+  # leading space and type $1
-                       (  # the type               $2
-                           [0-9]+   |
-                           [a-hA-H] |
-                           [ixvIXV]+  |
-                       )     
-                       \. # a single dot
-                       \s+)  # space
-                   (.*) # the string itself $3
-                  /sx) {
-        $self->type("li");
-        $self->removed($1);
-        $self->string($3);
-        $self->_identify_list_type($2); # this will set the type;
-        die "Something went wrong" if $self->type eq "null";
-    }
-    elsif ($l =~ m/^(\> )(.*)/s) {
-        $self->string($2);
-        $self->removed($1);
-        $self->type("verse");
-    }
-    # autofix
-    elsif ($l =~ m/^(\>)$/s) {
-        $self->string("\n");
-        $self->removed(">");
-        $self->type("verse");
-    }
-
-    # TODO catch table only with a trailing space. This needs a
-    # migration script, a Changes entry and a manual change. Too early
-    # for this.
-    elsif ($l =~ m/^(\s+)/ and $l =~ m/\|/) {
-        $self->type("table");
-        $self->string($l);
-    }
-    elsif ($l =~ m/^(\; (.+))$/s) {
-        $self->removed($l);
-        $self->type("comment");
-    }
-    elsif ($l =~ m/^((\[[0-9]+\])\s+)(.+)$/s) {
-        $self->type("footnote");
-        $self->string($3);
-        $self->removed($1);
-    }
-
-    elsif ($l =~ m/^((\s{6,})((\*\s?){5})\s*)$/s) {
-        $self->type("newpage");
-        $self->removed($2);
-        $self->string($3);
-    }
-
-    elsif ($l =~ m/^( {20,})([^ ].+)$/s) {
-        $self->block("right");
-        $self->type("regular");
-        $self->removed($1);
-        $self->string($2);
-    }
-    elsif ($l =~ m/^( {6,})([^ ].+)$/s) {
-        $self->block("center");
-        $self->type("regular");
-        $self->removed($1);
-        $self->string($2);
-    }
-    elsif ($l =~ m/^( {2,})([^ ].+)$/s) {
-        $self->block("quote");
-        $self->type("regular");
-        $self->removed($1);
-        $self->string($2);
-    }
-    else {
-        $self->type("regular");
-        $self->string($l);
-    }
-    # and finally assert that we didn't screw up
-    my $origline = $self->rawline;
-    my $test = $self->removed . $self->string;
-    die "We screw up: <$origline> ne <$test>\n"
-      unless $origline eq $test;
-}
-
-
-sub _identify_list_type {
-    my ($self, $list_type) = @_;
-    my $type;
-    if ($list_type =~ m/[0-9]/) {
-        $type = "oln";
-    } elsif ($list_type =~ m/[a-h]/) {
-        $type = "ola";
-    } elsif ($list_type =~ m/[A-H]/) {
-        $type = "olA";
-    } elsif ($list_type =~ m/[ixvl]/) {
-        $type = "oli";
-    } elsif ($list_type =~ m/[IXVL]/) {
-        $type = "olI";
-    } else {
-        die "$type Unrecognized, fix your code\n";
-    }
-    $self->block($type);
-}
-
 =head2 HELPERS
 
 =head3 is_start_block($blockname)
@@ -442,4 +292,53 @@ sub add_to_string {
     $self->string(join("", $orig, @args));
 }
 
+=head3 append($element)
+
+Append the element passed as argument to this one, setting th raw_line
+
+=cut
+
+sub append {
+    my ($self, $element) = @_;
+    $self->{rawline} .= $element->rawline;
+    my $type = $self->type;
+    if ($type eq 'example' or $type eq 'verse') {
+        $self->{string} .= $element->rawline;
+    }
+    else {
+        $self->{string} .= $element->string;
+    }
+
+}
+
+=head3 can_append($element)
+
+=cut
+
+sub can_append {
+    my ($self, $element) = @_;
+    if ($self->can_merge_next && $element->can_be_merged) {
+        return 1;
+    }
+    # same type
+    foreach my $type (qw/table versep null/) {
+        if ($self->type eq $type and $element->type eq $type) {
+            return 1;
+        }
+    }
+    return 0;
+}
+
+=head3 become_regular
+
+Set block to empty string and type to regular
+
+=cut
+
+sub become_regular {
+    my $self = shift;
+    $self->type('regular');
+    $self->block('');
+}
+
 1;
